import { of } from 'rxjs';
import { map, catchError, exhaustMap } from 'rxjs/operators';

import { Injectable } from '@angular/core';

import { Actions, createEffect, ofType } from '@ngrx/effects';

import { AuthResponse } from '@ucap/protocol-query';

import { QueryProtocolService } from '@ucap/ng-protocol-query';

import { auth, authSuccess, authFailure } from './actions';

@Injectable()
export class Effects {
  auth$ = createEffect(() =>
    this.actions$.pipe(
      ofType(auth),
      map(action => action.req),
      exhaustMap(req => {
        return this.queryProtocolService.auth(req).pipe(
          map((res: AuthResponse) => {
            return authSuccess({
              res
            });
          }),
          catchError(error => of(authFailure({ error })))
        );
      })
    )
  );

  constructor(
    private actions$: Actions,
    private queryProtocolService: QueryProtocolService
  ) {}
}
