import { of } from 'rxjs';
import { catchError, exhaustMap, map, switchMap } from 'rxjs/operators';

import { Injectable } from '@angular/core';

import { Actions, ofType, createEffect } from '@ngrx/effects';

import { Login2Response } from '@ucap/pi';

import { PiService } from '@ucap/ng-pi';
import { AuthenticationProtocolService } from '@ucap/ng-protocol-authentication';

import {
  logout,
  webLogin,
  webLoginSuccess,
  webLoginFailure,
  logoutSuccess
} from './actions';

@Injectable()
export class Effects {
  webLogin$ = createEffect(() =>
    this.actions$.pipe(
      ofType(webLogin),
      map(action => action),
      exhaustMap(
        (params: {
          loginId: string;
          loginPw: string;
          companyCode: string;
          rememberMe: boolean;
          autoLogin: boolean;
        }) =>
          this.piService
            .login2({
              loginId: params.loginId,
              loginPw: params.loginPw,
              companyCode: params.companyCode
            })
            .pipe(
              map((res: Login2Response) => {
                if ('success' !== res.status.toLowerCase()) {
                  return webLoginFailure({ error: 'Failed' });
                } else {
                  return webLoginSuccess({
                    loginId: params.loginId,
                    loginPw: params.loginPw,
                    companyCode: params.companyCode,
                    rememberMe: params.rememberMe,
                    autoLogin: params.autoLogin,
                    login2Response: res
                  });
                }
              }),
              catchError(error => of(webLoginFailure({ error })))
            )
      )
    )
  );

  logout$ = createEffect(() => {
    return this.actions$.pipe(
      ofType(logout),
      switchMap(action => {
        return this.authenticationProtocolService.logout({}).pipe(
          map(res => logoutSuccess({ res })),
          catchError(error => of(error))
        );
      })
    );
  });

  constructor(
    private actions$: Actions,
    private piService: PiService,
    private authenticationProtocolService: AuthenticationProtocolService
  ) {}
}
