import { createAction, props } from '@ngrx/store';
import { GroupRequest, GroupDetailData } from '@ucap/protocol-sync';
import {
  AddResponse as GroupAddResponse,
  UpdateRequest as GroupUpdateRequest,
  UpdateResponse as GroupUpdateResponse,
  DelRequest as GroupDelRequest,
  DelResponse as GroupDelResponse
} from '@ucap/protocol-group';

/**
 * retrieve list of group
 */
export const groups = createAction(
  '[ucap::group::group] groups',
  props<{ req: GroupRequest }>()
);
/**
 * Success of groups request
 */
export const groupsSuccess = createAction(
  '[ucap::group::group] groups Success',
  props<{ groupList: GroupDetailData[]; syncDate: string }>()
);
/**
 * Failure of groups request
 */
export const groupsFailure = createAction(
  '[ucap::group::group] groups Failure',
  props<{ error: any }>()
);

/**
 * 그룹원 수정
 */
export const updateMember = createAction(
  '[ucap::group::group] updateMember',
  props<{
    targetGroup: GroupDetailData;
    targetUserSeqs: string[];
  }>()
);
/**
 * Success of updateMember request
 */
export const updateMemberSuccess = createAction(
  '[ucap::group::group] updateMember Success',
  props<{
    targetGroup: GroupDetailData;
    targetUserSeqs: string[];
    userSeqsForDelete?: string[];
  }>()
);
/**
 * Failure of updateMember request
 */
export const updateMemberFailure = createAction(
  '[ucap::group::group] updateMember Failure',
  props<{ error: any }>()
);

/**
 * 그룹원 이동
 */
export const moveMember = createAction(
  '[ucap::group::group] moveMember',
  props<{
    fromGroup: GroupDetailData;
    toGroup: GroupDetailData;
    targetUserSeq: string[];
  }>()
);
/**
 * Failure of moveTo request
 */
export const moveToFailure = createAction(
  '[ucap::group::group] moveTo Failure',
  props<{ error: any }>()
);
/**
 * Failure of moveFrom request
 */
export const moveFromFailure = createAction(
  '[ucap::group::group] moveFrom Failure',
  props<{ error: any }>()
);

/**
 * 새그룹 추가
 */
export const create = createAction(
  '[ucap::group::group] create',
  props<{
    groupName: string;
    targetUserSeqs?: string[];
  }>()
);
/**
 * Success of create request
 */
export const createSuccess = createAction(
  '[ucap::group::group] create Success',
  props<{ res: GroupAddResponse; targetUserSeqs?: string[] }>()
);
/**
 * Failure of create request
 */
export const createFailure = createAction(
  '[ucap::group::group] create Failure',
  props<{ error: any }>()
);

/** 그룹 업데이트 */
export const update = createAction(
  '[ucap::group::group] update',
  props<{ req: GroupUpdateRequest }>()
);
/**
 * Success of update request
 */
export const updateSuccess = createAction(
  '[ucap::group::group] update Success',
  props<{ res: GroupUpdateResponse }>()
);
/**
 * Failure of update request
 */
export const updateFailure = createAction(
  '[ucap::group::group] update Failure',
  props<{ error: any }>()
);

/**
 * 그룹 삭제
 */
export const del = createAction(
  '[ucap::group::group] del',
  props<{
    group: GroupDetailData;
  }>()
);
/**
 * Success of del request
 */
export const delSuccess = createAction(
  '[ucap::group::group] del Success',
  props<{ res: GroupDelResponse }>()
);
/**
 * Failure of del request
 */
export const delFailure = createAction(
  '[ucap::group::group] del Failure',
  props<{ error: any }>()
);
